# ==================================================================
# Gossamer Threads Module Library - http://gossamer-threads.com/
#
#   GT::Plugins::Profiler
#   Author  : Alex Krohn
#   CVS Info :                     
#   $Id: Profiler.pm,v 1.5 2004/01/13 01:35:18 jagerman Exp $
#
# Copyright (c) 2004 Gossamer Threads Inc.  All Rights Reserved.
# ==================================================================
#
# Description: A plugin system for CGI scripts.
#

package GT::Plugins::Profiler;
# ==================================================================
use strict;
use vars    qw/@ISA $PROFILER $DEBUG $VERSION %HOOKS/;
use GT::Base;

@ISA     = qw/GT::Base/;
$DEBUG   = 0;
$VERSION = sprintf "%d.%03d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/;

sub profile {
# -----------------------------------------------------------------
# Analyses the plugin hook.
#
    my $class = shift;
    $PROFILER ||= $class->new;
    my ($plugin_obj, $hook_name, $input, $output) = @_;
    my $path = $plugin_obj->directory;

# Get the input types.
    my @input;
    for my $e (@$input) {
        push @input, (ref $e || 'SCALAR');
    }

# Get the output types.
    my @output;
    for my $e (@$output) {
        push @output, (ref $e || 'SCALAR');
    }

# find out from where this is being called
    my ($package, $filename, $line, $subroutine, 
        $hasargs, $wantarray, $evaltext, 
        $is_require, $hints, $bitmask) = caller(2);

    my $data = {
        hook_name     => $hook_name,
        path          => $path,
        input_params  => \@input,
        output_params => \@output,
        filename      => $filename,
        line          => $line,
        subroutine    => $subroutine,
        description   => '',
    };
    my $log = 1;
    
    $HOOKS{$path} ||= do "$path/hooks.cfg";
    for my $e (@{$HOOKS{$path}->{$hook_name}}) {
        if (not _is_different($e, $data)) {
            $log = 0;
            last;
        }
    }
    $log and push @{$HOOKS{$path}->{$hook_name}}, $data;
}

sub _is_different {
# -----------------------------------------------------------------
# Check to see if a structure is different.
#
    my ($first, $second) = @_;
    return 1 if $first xor $second;
    return 1 if ref $first ne ref $second;
    if (ref $first eq 'HASH') {
        for my $key (keys %$a) {
            return 1 if not ref $first->{$key} and $first->{$key} ne $second->{$key};
        }
    }
    elsif (ref $first eq 'ARRAY') {
        return 1 unless @$first == @$second;
        for my $i (0 .. $#$a) {
            if (ref $a->[$i]) {
                return 1 if _is_different($a->[$i], $b->[$i]);
            }
            else {
                return 1 if $a->[$i] ne $b->[$i];
            }
        }
    }
    else {
        return $a ne $b;          
    }
    return 0;
}

sub DESTROY {
# -----------------------------------------------------------------
    for my $path (keys %HOOKS) {
        open PLUGIN_FILE, ">$path/hooks.cfg" or die "can't open: $path/hooks.cfg ($!)";
        require GT::Dumper;
        print PLUGIN_FILE GT::Dumper->dump(var => '', data => $HOOKS{$path});
        close PLUGIN_FILE;
    }
}

1;

__END__

=head1 NAME

GT::Plugins::Profiler - analyze what hooks you have in a program.

=head1 SYNOPSIS

    $GT::Plugins::PROFILE = 1;

=head1 DESCRIPTION

This module will create a hooks.cfg file that contains a perl dump
of all the hooks found during execution of a program.

=head1 COPYRIGHT

Copyright (c) 2004 Gossamer Threads Inc.  All Rights Reserved.
http://www.gossamer-threads.com/

=head1 VERSION

Revision: $Id: Profiler.pm,v 1.5 2004/01/13 01:35:18 jagerman Exp $

=cut
