# ====================================================================
# Gossamer Threads Module Library - http://gossamer-threads.com/
#
#   GT::WWW::http
#   Author: Jason Rhinelander
#   $Id: https.pm,v 1.3 2004/01/13 01:35:20 jagerman Exp $
#
# Copyright (c) 2004 Gossamer Threads Inc.  All Rights Reserved.
# ====================================================================
#
# Description:
#   GT::WWW::http subclass to handle HTTPS connections
#
# This class has only one methods of its own - the default port. Everything
# else is inherited directly from GT::WWW::http.  It does, however, have the
# SSLHandle use, which will err fatally if Net::SSLeay is not installed.

package GT::WWW::https;
use GT::WWW::http;
use GT::Socket::Client::SSLHandle;

@GT::WWW::https::ISA = 'GT::WWW::http';

sub default_port { 443 }

1;

__END__

=head1 NAME

GT::WWW::https - HTTPS handling for GT::WWW

=head1 DESCRIPTION

This module is a simple subclass of GT::WWW::http used by GT::WWW to enable
HTTPS access as opposed to HTTP access.  Thus GT::WWW::http should be consulted
instead of this documentation.

=head1 REQUIREMENTS

GT::WWW HTTPS support requires GT::Socket::Client::SSLHandle, which in turn
requires the Net::SSLeay library.

=head1 SEE ALSO

L<GT::WWW::http>

=head1 MAINTAINER

Jason Rhinelander

=head1 COPYRIGHT

Copyright (c) 2004 Gossamer Threads Inc.  All Rights Reserved.
http://www.gossamer-threads.com/

=head1 VERSION

Revision: $Id: https.pm,v 1.3 2004/01/13 01:35:20 jagerman Exp $

=cut
